package org.jooq.util;

import org.h2.jdbcx.JdbcDataSource;
import org.hibernate.cfg.Configuration;
import org.hibernate.cfg.Environment;
import org.hibernate.dialect.H2Dialect;
import org.hibernate.tool.hbm2ddl.SchemaExport;
import org.hibernate.tool.hbm2ddl.Target;
import org.jooq.util.h2.H2Database;

import java.io.File;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.List;

public class H2Db {
  private H2Database db = new H2Database();
  private List<Class<?>> types;
  private Connection connection;
  public H2Db(List<Class<?>> types) {
    this.types = types;

    connection = getConnection();
    db.setConnection(connection);

    export();
  }

  public H2Database getDb() {
    return db;
  }

  private void export() {
    Configuration configuration = new Configuration().setProperty(
        "hibernate.dialect", "org.hibernate.dialect.H2Dialect"
    );

    for(Class<?> type : types) {
      configuration.addAnnotatedClass(type);
    }

    SchemaExport export = new SchemaExport(configuration, connection);
    export.create(Target.EXPORT);
  }

  private Connection getConnection() {
    JdbcDataSource ds = new JdbcDataSource();
    ds.setURL("jdbc:h2:mem:codegen\\;init=create schema codegen\\;set schema codegen;DATABASE_TO_UPPER=false");
    try {
      return ds.getConnection();
    } catch (SQLException e) {
      throw new RuntimeException(e);
    }
  }
}