package org.jooq.util;

import com.here.analytics.insights.persistence.core.CodeGenerationConfig;
import org.jooq.util.h2.H2Database;
import org.jooq.util.jaxb.*;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.context.annotation.ClassPathScanningCandidateComponentProvider;
import org.springframework.core.type.filter.AnnotationTypeFilter;
import org.springframework.core.type.filter.AssignableTypeFilter;

import javax.persistence.Entity;
import java.lang.reflect.Constructor;
import java.util.ArrayList;
import java.util.List;

public class JpaCodeGenerator extends AbstractGenerator {
  private JavaGenerator delegate = new JavaGenerator();

  public JpaCodeGenerator() {
    DefaultGeneratorStrategy s = new DefaultGeneratorStrategy();
    setStrategy(s);
    delegate.setStrategy(s);
  }

  @Override
  public void generate(Database database) {
    //doesn't matter, completely ignore database, go out to any sub modules and scan their class paths
    //there's not enough context to really do anything clever here, so we'll brute force, scan the classpath for marker interfaces
    //then use the paths and packages they specify to hoist up a h2 instance, populate it, then generate the classes,
    //once they are generated, go through and generate repositories for all the classes
    for(CodeGenerationConfig config : configs()) {
      for(String pkg : config.getIncludedPackages()) {
        List<Class<?>> entityClasses = allEntities(new String[] { pkg });

        H2Database db = new H2Db(entityClasses).getDb();

        delegate.setTargetDirectory(config.getExportPath());
        delegate.setTargetPackage(pkg);

        db.setIncludes(new String[] { ".*" });
        db.setExcludes(new String[0]);
        db.setRecordVersionFields(new String[0]);
        db.setRecordTimestampFields(new String[0]);
        db.setSchemaVersionProvider(new ConstantSchemaVersionProvider("1"));
        db.setConfiguredEnumTypes(new ArrayList<EnumType>());
        db.setConfiguredForcedTypes(new ArrayList<ForcedType>());
        db.setConfiguredCustomTypes(new ArrayList<CustomType>());

        List<Schema> schemas = new ArrayList<Schema>();
        schemas.add(new Schema().withInputSchema("codegen").withOutputSchema("Generated"));

        db.setConfiguredSchemata(schemas);

        delegate.generate(db);
      }
    }
  }

  private List<CodeGenerationConfig> configs() {
    ClassPathScanningCandidateComponentProvider provider = new ClassPathScanningCandidateComponentProvider(false);
    provider.addIncludeFilter(new AssignableTypeFilter(CodeGenerationConfig.class));
    List<CodeGenerationConfig> allConfigs = new ArrayList<CodeGenerationConfig>();

    for(BeanDefinition definition : provider.findCandidateComponents(getTargetPackage())) {
      String className = definition.getBeanClassName();
      try {
        Constructor c = Class.forName(className).getConstructor();
        allConfigs.add((CodeGenerationConfig)c.newInstance());
      } catch (Exception e) {
        throw new RuntimeException(e);
      }
    }

    return allConfigs;
  }

  private List<Class<?>> allEntities(String[] packages) {
    ClassPathScanningCandidateComponentProvider provider = new ClassPathScanningCandidateComponentProvider(false);
    provider.addIncludeFilter(new AnnotationTypeFilter(Entity.class));
    List<Class<?>> entityClasses = new ArrayList<Class<?>>();
    for (String pkg : packages) {
      for (BeanDefinition definition : provider.findCandidateComponents(pkg)) {
        try {
          entityClasses.add(Class.forName(definition.getBeanClassName()));
        } catch (ClassNotFoundException e) {
          throw new RuntimeException(e);
        }
      }
    }
    return entityClasses;
  }
}